import { Component, EventEmitter, Input, Output, OnInit } from "@angular/core";
import { Pipe, PipeTransform } from "@angular/core";
import { Translation, TranslationService, Language } from "angular-l10n";
import * as moment from "moment";
import { ChsService } from "../chs.service";
import { Device } from "../devices/device";
import { strSetChar } from "../utils";


@Pipe({ name: "TypeToIndex" })
export class TypeToIndexPipe implements PipeTransform {
	private static _TypeToIndex = { "static": 0, "random": 1, "astroSunrise": 2, "astroSunset": 3 };
	transform(value: string): number {
		return TypeToIndexPipe._TypeToIndex[value];
	}
}

@Pipe({ name: "TargetToIndex" })
export class TargetToIndexPipe implements PipeTransform {
	private static _TargetToIndex = { "driveUp": 0, "drivePos2": 1, "drivePos1": 2, "driveDown": 3, "drivePos ": 4 };
	transform(value: string): number {
		return TargetToIndexPipe._TargetToIndex[value];
	}
}

@Pipe({ name: "TimeToDate" })
export class TimeToDatePipe implements PipeTransform {
	transform(value: string): Date {
		if (!value) { return new Date(1970, 1, 1, 12, 0, 0); }
		let splitted: string[] = value.split(":");
		if (splitted.length !== 2) { return new Date(1970, 1, 1, 12, 0, 0); }

		return new Date(1970, 1, 1, parseInt(splitted[0], 10), parseInt(splitted[1], 10), 0);
	}
}
@Component({
	moduleId: module.id,
	selector: "timer-editor",
	templateUrl: "timereditor.component.html"
})
export class TimereditorComponent implements OnInit {
	@Input() public timer: Device;
	@Input() public devType: string;
	@Output() onDeleted = new EventEmitter<string>();
	@Language() lang: string;

	private _targetState: string;
	private _targetPos: number;

	private _sunday: boolean;
	private _monday: boolean;
	private _tuesday: boolean;
	private _wednesday: boolean;
	private _thursday: boolean;
	private _friday: boolean;
	private _saturday: boolean;

	private static _IndexToType = { 0: "static", 1: "random", 2: "astroSunrise", 3: "astroSunset" };
	private _dropDownTypeSource = ["individuell", "Zufall", "Astro morgens", "Astro abends"];
	private _dropDownTargetSource = [ { label: "", target: "driveUp" }, { label: "", target: "drivePos2" },
																			{ label: "", target: "drivePos1" }, { label: "", target: "driveDown" }, { label: "", target: "drivePos " } ];

	constructor(public translation: TranslationService, private _chsService: ChsService) {
		 
	}

	onClose() {
		this.onDeleted.emit(this.timer.id);
	}

	private setDays(days: string): void {
		this._sunday = (days[0] === "1");
		this._monday = (days[1] === "1");
		this._tuesday = (days[2] === "1");
		this._wednesday = (days[3] === "1");
		this._thursday = (days[4] === "1");
		this._friday = (days[5] === "1");
		this._saturday = (days[6] === "1");
	}

	private daysToArray(days: string): number[] {
		if (!days || days.length !== 7) { return; }
		let retVal: number[] = [];
		let i: number = 0;
		while (i < 7) {
			if (days[i] === "1") {
				retVal.push(i);
			}
			++i;
		}
		return retVal;
	}

	ngOnInit() {
		if (this.timer.time==null) {
			this.timer.time="12:00";
		}
	
		this._dropDownTypeSource[0]=this.translation.translate("TIMERTINDIVIDUAL");
		this._dropDownTypeSource[1]=this.translation.translate("TIMERTRANDOM");
		this._dropDownTypeSource[2]=this.translation.translate("TIMERTASTROM");
		this._dropDownTypeSource[3]=this.translation.translate("TIMERTASTROE");
		//this._dropDownTypeSource.placeHolder=this.translation.translate("CHOOSETIMERT");

		let dtTargets: any = this._dropDownTargetSource;
		dtTargets[0].label=this.translation.translate("POSUP");
		dtTargets[1].label=this.translation.translate("POSAIR");
		dtTargets[2].label=this.translation.translate("POSINT");
		dtTargets[3].label=this.translation.translate("POSDOWN");
		dtTargets[4].label=this.translation.translate("POSIND");

		if (this.devType === "slider") {
			this._targetPos = 50;
			this._targetState = this.timer.device_state;
			if (this._targetState.startsWith("drivePos ")) {
				// this._targetPos=Number.parseInt(this._targetState.substring(8));
				this._targetPos = Number(this._targetState.substring(9));
				this._targetState = this._targetState.substring(0, 9);
			}
		}	else if (this.devType === "onoff") {
			this._targetState = this.timer.device_state;
		}
		this.setDays(this.timer.days);
	}

	onDayChange(event: any) {
		let name = event.target.name;
		let checked = event.target["checked"] ? "1" : "0";
		let index = -1;
		switch (name) {
			case "_sunday": index = 0; break;
			case "_monday": index = 1; break;
			case "_tuesday": index = 2; break;
			case "_wednesday": index = 3; break;
			case "_thursday": index = 4; break;
			case "_friday": index = 5; break;
			case "_saturday": index = 6; break;
		}

		this.timer.days = strSetChar(this.timer.days, index, checked);
		this.saveTimer();
	}

	onNotBeforeAfterChange(event: any): void {
		let checked: any = event.target["checked"];
		if (!checked) {
			this.timer.time = null;
		} else {
			this.timer.time = "12:00";
		}
		this.saveTimer();
	}

	onTimeChange(event: any): void {
		var d: Date = event.args.date;
		let m = moment(d).utc().local();
		let strTime = m.format("HH:mm");
		if (this.timer.time !== strTime) {
			this.timer.time = strTime;
			this.saveTimer();
		}
	}

	onTypeChanged(event: any): void {
		var index=event.args.index;
		this.timer.timer_type = TimereditorComponent._IndexToType[index];
		this.saveTimer();
	}

	onSwitchChange(val: any): void {
		let strNewVal = (val.args.checked ? "on" : "off");
		if (this.timer.state !== strNewVal) {
			this.timer.state = strNewVal;
			this.saveTimer();
		}
	}

	onOffsetChange(event: any): void {
		let val: number = event.args.value;
		var num: number = Math.round(val);
		if (this.timer.offset !== num) {
			this.timer.offset = num;
			this.saveTimer();
		}
	}


	private saveTimer(): void {
		this._chsService.writeDevice(this.timer).subscribe();
	}

	onTargetPosChanged(args: any): void {
		let pos: number = args.value;
		if (pos !== this._targetPos) {
			this._targetPos = pos;
			this.timer.device_state = "drivePos " + this._targetPos.toString();
			this.saveTimer();
		}
	}

	private onChangeTarget(event: any): void {
		//let index: number = event.args.index;
		//this._targetState = this._dropDownTargetSource[index].target;
		this._targetState = event;
		this.timer.device_state = this._targetState;
		if (this._targetState === "drivePos ") {
			this.timer.device_state = "drivePos " + this._targetPos.toString();
		}
		this.saveTimer();
	}
}